#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
fetchez.modules
~~~~~~~~~~~~~~~~~~~~~~

modules init

:copyright: (c) 2010-2026 Regents of the University of Colorado
:license: MIT, see LICENSE for more details.
"""

from .base import FetchModule

__all__ = ["FetchModule"]
